import {Component, ViewEncapsulation, ViewChild} from '@angular/core';
import {DiagramComponent} from '@syncfusion/ej2-angular-diagrams';
import {
    Diagram, NodeModel, UndoRedo, ConnectorModel, GradientModel, LinearGradientModel, PaletteModel, SymbolInfo
} from '@syncfusion/ej2-diagrams';
import {aiFlowService} from '../ai-flow.service';
import {event} from 'firebase-functions/lib/providers/analytics';
import {MatSnackBar} from '@angular/material';
import { Toolbar, ClickEventArgs, ItemModel } from '@syncfusion/ej2-navigations';
import { openPalette } from '../diagram.common';

Diagram.Inject(UndoRedo);

@Component({
    selector: 'app-flowdiagram',
    templateUrl: './flowdiagram.component.html',
    styleUrls: ['./flowdiagram.component.css']
})
export class FlowdiagramComponent {
    @ViewChild('diagram', null)
    public diagramm: DiagramComponent;
    private jsonOutput: any;
    private connectors: any;
    private finalJson: any;
    private shapes:NodeModel[]=[];
    private diagram: Diagram;

    constructor(private aiflowservice: aiFlowService, public snackBar: MatSnackBar) {
    }

    // define nodes
    public nodes: NodeModel[] = [
        {
            id: 'nodeA', width: 200, height: 30, offsetX: 200, offsetY: 200, style: {fill: '#726dd1', strokeColor: '#726dd1'}
        },
        {
            id: 'nodeB', width: 200, height: 100, offsetX: 200, offsetY: 250, style: {fill: '#dbe6eb', strokeColor: '#dbe6eb'},
            annotations: [{content: 'Are you ready to register? You can do so right here', style: {fontSize: 14}}]
        },
        //create group node 1
        {
            id: 'Ready to Register', children: ['nodeA', 'nodeB']
        },
        {
            id: 'Yes I am',
            annotations: [
                {
                    content: 'Yes I am'
                }
            ],
            shape: {type: 'Flow', shape: 'Terminator'},
            style: {fill: '#b260b5', strokeColor: '#b260b5'}
        },
        {
            id: 'Question',
            annotations: [
                {
                    content: 'I have a question'
                }
            ],
            shape: {type: 'Flow', shape: 'Terminator'},
            style: {fill: '#726dd1', strokeColor: '#726dd1'}
        },
        {
            id: 'nodeC', width: 200, height: 30, offsetX: 200, offsetY: 200, style: {fill: '#b260b5', strokeColor: 'black'}
        },
        {
            id: 'nodeD', width: 200, height: 100, offsetX: 200, offsetY: 250, style: {fill: '#dbe6eb', strokeColor: '#dbe6eb'},
            annotations: [{content: 'What email should we register you under for the event', style: {fontSize: 14}}]
        },
        //create group node 1
        {
            id: 'Email Capture', children: ['nodeC', 'nodeD']
        },
        {
            id: 'Process',
            width: 200,
            height: 300,
            offsetX: 200,
            offsetY: 300,
            style: {fill: '#dbe6eb', strokeColor: '#dbe6eb'},
            shape: {type: 'Flow', shape: 'Process'},
        },
        {
            id: 'nodeL',
            width: 150,
            height: 50,
            offsetX: 200,
            offsetY: 200,
            style: {fill: '#b260b5', strokeColor: '#b260b5'},
            shape: {type: 'Flow', shape: 'Terminator'},
            annotations: [{content: 'Will i be sent', style: {fontSize: 14}}]
        },
        {
            id: 'nodeM',
            width: 150,
            height: 50,
            offsetX: 200,
            offsetY: 270,
            style: {fill: '#b260b5', strokeColor: '#b260b5'},
            shape: {type: 'Flow', shape: 'Terminator'},
            annotations: [{content: 'What timezone', style: {fontSize: 14}}]
        },
        {
            id: 'nodeN',
            width: 150,
            height: 50,
            offsetX: 200,
            offsetY: 340,
            style: {fill: '#b260b5', strokeColor: '#b260b5'},
            shape: {type: 'Flow', shape: 'Terminator'},
            annotations: [{content: 'Can i register', style: {fontSize: 14}}]
        },
        {
            id: 'nodeO',
            width: 150,
            height: 50,
            offsetX: 200,
            offsetY: 410,
            style: {fill: '#b260b5', strokeColor: '#b260b5'},
            shape: {type: 'Flow', shape: 'Terminator'},
            annotations: [{content: 'Is this a virtual', style: {fontSize: 14}}]
        },
        //create group node
        {
            id: 'Group', children: ['Process', 'nodeL', 'nodeM', 'nodeN', 'nodeO']
        }
    ];

    // public connectors: ConnectorModel[] = [
    //   {
    //     id: 'connector',
    //     sourceID: 'WhatisJAVA',
    //     targetID: 'ExplainOOPS'
    //   }
    // ];

    // Initialize the flowshapes for the symbol palatte
    private flowshapes: NodeModel[] = [
        {
            id: 'nodeA', width: 200, height: 30, offsetX: 200, offsetY: 200, style: {fill: '#726dd1', strokeColor: 'black'}
        },
        {
            id: 'nodeB', width: 200, height: 100, offsetX: 200, offsetY: 250, style: {fill: '#dbe6eb', strokeColor: '#dbe6eb'},
            annotations: [{content: 'Are you ready to register? You can do so right here', style: {fontSize: 14}}]
        },
        //create group node 1
        {
            id: 'Ready to Register', children: ['nodeA', 'nodeB']
        },
        {
            id: 'Yes I am',
            annotations: [
                {
                    content: 'Yes I am'
                }
            ],
            shape: {type: 'Flow', shape: 'Terminator'},
            style: {fill: '#b260b5', strokeColor: '#b260b5'}
        },
        {
            id: 'Question',
            annotations: [
                {
                    content: 'I have a question'
                }
            ],
            shape: {type: 'Flow', shape: 'Terminator'},
            style: {fill: '#726dd1', strokeColor: '#726dd1'}
        },
        {
            id: 'nodeC', width: 200, height: 30, offsetX: 200, offsetY: 200, style: {fill: '#b260b5', strokeColor: 'black'}
        },
        {
            id: 'nodeD', width: 200, height: 100, offsetX: 200, offsetY: 250, style: {fill: '#dbe6eb', strokeColor: '#dbe6eb'},
            annotations: [{content: 'What email should we register you under for the event', style: {fontSize: 14}}]
        },
        //create group node 1
        {
            id: 'Email Capture', children: ['nodeC', 'nodeD']
        },
        {
            id: 'Process',
            width: 200,
            height: 300,
            offsetX: 200,
            offsetY: 300,
            style: {fill: '#dbe6eb', strokeColor: '#dbe6eb'},
            shape: {type: 'Flow', shape: 'Process'},
        },
        {
            id: 'nodeL',
            width: 150,
            height: 50,
            offsetX: 200,
            offsetY: 200,
            style: {fill: '#b260b5', strokeColor: '#b260b5'},
            shape: {type: 'Flow', shape: 'Terminator'},
            annotations: [{content: 'Will i be sent', style: {fontSize: 14}}]
        },
        {
            id: 'nodeM',
            width: 150,
            height: 50,
            offsetX: 200,
            offsetY: 270,
            style: {fill: '#b260b5', strokeColor: '#b260b5'},
            shape: {type: 'Flow', shape: 'Terminator'},
            annotations: [{content: 'What timezone', style: {fontSize: 14}}]
        },
        {
            id: 'nodeN',
            width: 150,
            height: 50,
            offsetX: 200,
            offsetY: 340,
            style: {fill: '#b260b5', strokeColor: '#b260b5'},
            shape: {type: 'Flow', shape: 'Terminator'},
            annotations: [{content: 'Can i register', style: {fontSize: 14}}]
        },
        {
            id: 'nodeO',
            width: 150,
            height: 50,
            offsetX: 200,
            offsetY: 410,
            style: {fill: '#b260b5', strokeColor: '#b260b5'},
            shape: {type: 'Flow', shape: 'Terminator'},
            annotations: [{content: 'Is this a virtual', style: {fontSize: 14}}]
        },
        //create group node
        {
            id: 'Group', children: ['Process', 'nodeL', 'nodeM', 'nodeN', 'nodeO']
        }
    ];

    private connectorSymbols: ConnectorModel[] = [
        {
            id: 'Link1',
            type: 'Orthogonal',
            sourcePoint: {x: 0, y: 0},
            targetPoint: {x: 60, y: 60},
            targetDecorator: {shape: 'Arrow'},
            style: {strokeWidth: 1}
        },
        {
            id: 'link3',
            type: 'Orthogonal',
            sourcePoint: {x: 0, y: 0},
            targetPoint: {x: 60, y: 60},
            style: {strokeWidth: 1},
            targetDecorator: {shape: 'None'}
        },
        {
            id: 'Link21',
            type: 'Straight',
            sourcePoint: {x: 0, y: 0},
            targetPoint: {x: 60, y: 60},
            targetDecorator: {shape: 'Arrow'},
            style: {strokeWidth: 1}
        },
        {
            id: 'link23',
            type: 'Straight',
            sourcePoint: {x: 0, y: 0},
            targetPoint: {x: 60, y: 60},
            style: {strokeWidth: 1},
            targetDecorator: {shape: 'None'}
        },
        {
            id: 'link33',
            type: 'Bezier',
            sourcePoint: {x: 0, y: 0},
            targetPoint: {x: 60, y: 60},
            style: {strokeWidth: 1},
            targetDecorator: {shape: 'None'}
        }
    ];

    public palettes: PaletteModel[] = [
        {
            id: 'flow',
            expanded: true,
            symbols: this.flowshapes,
            iconCss: 'shapes',
            title: 'Flow Shapes'
        },
        {
            id: 'connectors',
            expanded: true,
            symbols: this.connectorSymbols,
            iconCss: 'shapes',
            title: 'Connectors'
        }
    ];

    openSnackBar(message: string, action: string) {
        this.snackBar.open(message, action, {
            duration: 3000,
        });
    }


    public onClicked() {
        this.jsonOutput = JSON.parse(this.diagramm.saveDiagram());
        this.connectors = this.jsonOutput.connectors;
        this.nodes = this.jsonOutput.nodes;

        for(let i=0;i<this.nodes.length;i++){
            let id = this.nodes[i].id;
            if(id.substring(0, 4)!='node'){
                this.shapes.push(this.nodes[i]);
            }
        }
        console.log("this.shapes", this.shapes);
        this.finalJson = this.shapes.concat(this.connectors);
        console.log('Final JSON==========>', this.finalJson);

        // this.download(this.json);

        this.aiflowservice.postFlowdata(localStorage.getItem('userId'),this.finalJson).subscribe(
            event => {
                let responseBody = event;
                if (responseBody.status = 'success') {
                    this.openSnackBar('Flow is created', 'Ok');
                } else {
                    this.openSnackBar('responseBody.message', 'Ok');
                }
            },
            (err) => {
                this.openSnackBar('Unknown error, please try again later', 'Ok');
                console.log('Upload Error:', err);
            }, () => {
                console.log('Upload done');
            }
        );
    }

    public getdata(){
      this.openSnackBar('Request Sent', 'Ok');
      this.aiflowservice.getFlowData(localStorage.getItem('userId')).subscribe(
          event =>{
            console.log("event",JSON.stringify(event));
          }
      )
    }

    public dragEnter(args: any): void {
        args.element.width = 150;
        args.element.height = 60;
        console.log('dragEnter is called', args);
    }

    public getSymbolInfo(symbol: NodeModel): SymbolInfo {
        // display text for palette item
        return {description: {text: symbol.id}, fit: true};
        console.log('getSymbolInfo', symbol);
    }

    public textEdit(value: any) {
        console.log('<<-----------Text is added-------->>');
        if (value.oldValue) {
            console.log('Old Value-------->>', value.oldValue);
        }
        console.log('New Value-------->>', value.newValue);
    }

    // loadDiagram(event: ProgressEvent): void {
    //     this.diagram.loadDiagram((event.target as FileReader).result.toString());
    // }
    //
    // public onUploadSuccess(args: { [key: string]: Object }): void {
    //     let file1: { [key: string]: Object } = args.file as { [key: string]: Object };
    //     let file: Blob = file1.rawFile as Blob;
    //     let reader: FileReader = new FileReader();
    //     reader.readAsText(file);
    //     reader.onloadend = this.loadDiagram;
    // }
    //
    // toolbarClick(args: ClickEventArgs): void {
    //     if (args.item.text === 'New') {
    //         this.diagram.clear();
    //     } else if (args.item.text === 'Load') {
    //         document.getElementsByClassName('e-file-select-wrap')[0].querySelector('button').click();
    //     } else if (args.item.id === 'palette-icon') {
    //         openPalette();
    //     } else {
    //         this.download(this.diagram.saveDiagram());
    //     }
    // }
    //
    // download(data: string): void {
    //     if (window.navigator.msSaveBlob) {
    //         let blob: any = new Blob([data], { type: 'data:text/json;charset=utf-8,' });
    //         window.navigator.msSaveOrOpenBlob(blob, 'Diagram.json');
    //     } else {
    //         let dataStr: string = 'data:text/json;charset=utf-8,' + encodeURIComponent(data);
    //         let a: HTMLAnchorElement = document.createElement('a');
    //         a.href = dataStr;
    //         a.download = 'Diagram.json';
    //         document.body.appendChild(a);
    //         a.click();
    //         a.remove();
    //     }
    // }


}
